from django.contrib.auth import get_user_model, update_session_auth_hash
from django.contrib.auth.models import Group
from django.core.paginator import Paginator, PageNotAnInteger, EmptyPage
from django.urls import reverse_lazy
from django.shortcuts import get_object_or_404
from django.views.generic import UpdateView, FormView
from django.views.generic.detail import DetailView
from django.db.models import Q

from braces.views import LoginRequiredMixin, SuperuserRequiredMixin

from rest_framework import viewsets
from rest_framework import filters
from rest_framework import generics
from rest_framework.response import Response

from django_filters.rest_framework import DjangoFilterBackend

from .forms import ProfileEditForm, ProfileEditAdminForm, AcceptTermsForm

from ..core.permissions import IsAdmin
from users.serializers import UserAdminSerializer, GroupAdminSerializer, GroupSerializer


User = get_user_model()


class ProfileEditView(LoginRequiredMixin, UpdateView):
    model = get_user_model()
    form_class = ProfileEditForm
    template_name = 'profile-edit.html'

    def get_success_url(self):
        url = self.request.GET.get('next', None)
        if url:
            return url
        else:
            return reverse_lazy('courses_legacy:profile')

    def get_object(self):
        return self.request.user

    def post(self, request, *args, **kwargs):
        form_result = super(ProfileEditView, self) \
            .post(request, *args, **kwargs)
        update_session_auth_hash(self.request, self.request.user)
        return form_result


class ProfileEditAdminView(LoginRequiredMixin, SuperuserRequiredMixin, UpdateView):
    model = get_user_model()
    form_class = ProfileEditAdminForm
    template_name = 'profile-edit.html'
    raise_exception = True

    def get_success_url(self):
        url = self.request.GET.get('next', None)
        if url:
            return url
        else:
            return reverse_lazy('courses_legacy:administration.profile_edit', args=[self.kwargs['username']])

    def get_object(self):
        if hasattr(self, 'kwargs') and 'username' in self.kwargs:
            try:
                return get_object_or_404(self.model, username=self.kwargs['username'])
            except:
                return self.request.user
        else:
            return self.request.user


class ProfileView(LoginRequiredMixin, DetailView):
    model = get_user_model()
    template_name = 'profile.html'
    context_object_name = 'profile_user'

    def get_object(self):
        if hasattr(self, 'kwargs') and 'username' in self.kwargs:
            try:
                return get_object_or_404(self.model, username=self.kwargs['username'])
            except:
                return self.request.user
        else:
            return self.request.user


class TimtecUserAdminViewSet(viewsets.ModelViewSet):
    model = get_user_model()
    # lookup_field = 'id'
    # filter_backends = (filters.OrderingFilter,)
    permission_classes = (IsAdmin, )
    serializer_class = UserAdminSerializer
    ordering = ('first_name', 'username',)
    queryset = User.objects.all()
    # search_fields = ('first_name', 'last_name', 'username', 'email')

    def get_queryset(self):
        page = self.request.query_params.get('page')
        keyword = self.request.query_params.get('keyword')
        admin = self.request.query_params.get('admin')
        blocked = self.request.query_params.get('blocked')
        queryset = super().get_queryset().order_by('first_name')

        if keyword:
            queryset = queryset.filter(Q(first_name__icontains=keyword) |
                                       Q(last_name__icontains=keyword) |
                                       Q(username__icontains=keyword) |
                                       Q(email__icontains=keyword))

        if admin == 'true':
            queryset = queryset.filter(is_superuser=True)

        if blocked == 'true':
            queryset = queryset.filter(is_active=False)

        if page:
            paginator = Paginator(queryset, 50)
            try:
                queryset = paginator.page(page)
            except PageNotAnInteger:
                # If page is not an integer, deliver first page.
                queryset = paginator.page(1)
            except EmptyPage:
                # If page is out of range (e.g. 9999),
                # deliver last page of results.
                queryset = paginator.page(paginator.num_pages)

        return queryset


class AcceptTermsView(FormView):
    template_name = 'accept-terms.html'
    form_class = AcceptTermsForm
    success_url = reverse_lazy('courses_legacy:courses')

    def get_success_url(self):
        next_url = self.request.POST.get('next', None)
        if next_url:
            return next_url
        return reverse_lazy('courses_legacy:courses')

    def form_valid(self, form):
        # This method is called when valid form data has been POSTed.
        # It should return an HttpResponse.
        self.request.user.accepted_terms = True
        self.request.user.save()
        return super(AcceptTermsView, self).form_valid(form)

    def get_context_data(self, **kwargs):
        context = super(AcceptTermsView, self).get_context_data(**kwargs)

        next_url = self.request.GET.get('next')
        if next_url:
            context['next_url'] = next_url
        return context
